'use strict';

function _arrayLikeToArray(r, a) {
  (null == a || a > r.length) && (a = r.length);
  for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e];
  return n;
}
function _arrayWithoutHoles(r) {
  if (Array.isArray(r)) return _arrayLikeToArray(r);
}
function _iterableToArray(r) {
  if ("undefined" != typeof Symbol && null != r[Symbol.iterator] || null != r["@@iterator"]) return Array.from(r);
}
function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}
function _toConsumableArray(r) {
  return _arrayWithoutHoles(r) || _iterableToArray(r) || _unsupportedIterableToArray(r) || _nonIterableSpread();
}
function _unsupportedIterableToArray(r, a) {
  if (r) {
    if ("string" == typeof r) return _arrayLikeToArray(r, a);
    var t = {}.toString.call(r).slice(8, -1);
    return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0;
  }
}

var orange = 'light-dark(rgb(232,98,0), rgb(255, 150, 50))';
var purple = 'light-dark( #881391, #D48CE6)';
var gray = 'light-dark(rgb(119,119,119), rgb(201, 201, 201))';
var listStyle = {
  style: 'list-style-type: none; padding: 0; margin: 0 0 0 12px; font-style: normal; position: relative'
};
var immutableNameStyle = {
  style: "color: ".concat(orange, "; position: relative")
};
var keyStyle = {
  style: "color: ".concat(purple)
};
var defaultValueKeyStyle = {
  style: "color: ".concat(gray)
};
var alteredValueKeyStyle = {
  style: "color: ".concat(purple, "; font-weight: bolder")
};
var inlineValuesStyle = {
  style: "color: ".concat(gray, "; font-style: italic; position: relative")
};
var nullStyle = {
  style: "color: ".concat(gray)
};
function createFormatters(Immutable) {
  var isRecord = function isRecord(maybeRecord) {
    if (maybeRecord && maybeRecord._values === undefined &&
    // in v3 record
    maybeRecord._map === undefined // in v4 record
    ) {
      // don't detect Immutable.Record.prototype as a Record instance
      return;
    }
    // Immutable v4
    if (maybeRecord['@@__IMMUTABLE_RECORD__@@']) {
      // There's also a Immutable.Record.isRecord we could use, but then the
      // Immutable instance passed into createFormatter has to be the same
      // version as the one used to create the Immutable object.
      // That's especially a problem for the Chrome extension.
      return true;
    }
    // Immutable v3
    return !!(maybeRecord['@@__IMMUTABLE_KEYED__@@'] && maybeRecord['@@__IMMUTABLE_ITERABLE__@@'] && maybeRecord._defaultValues !== undefined);
  };
  var reference = function reference(object, config) {
    if (typeof object === 'undefined') return ['span', nullStyle, 'undefined'];else if (object === null) return ['span', nullStyle, 'null'];
    return ['object', {
      object: object,
      config: config
    }];
  };
  var renderIterableHeader = function renderIterableHeader(iterable) {
    var name = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'Iterable';
    return ['span', ['span', immutableNameStyle, name], ['span', "[".concat(iterable.size, "]")]];
  };
  var getKeySeq = function getKeySeq(collection) {
    return collection.toSeq().map(function (v, k) {
      return k;
    }).toIndexedSeq();
  };
  var hasBody = function hasBody(collection, config) {
    return getKeySeq(collection).size > 0 && !(config && config.noPreview);
  };
  var renderIterableBody = function renderIterableBody(collection, mapper) {
    var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    if (options.sorted) {
      collection = collection.sortBy(function (value, key) {
        return key;
      });
    }
    var children = collection.map(mapper).toList();
    var jsList = [];
    children.forEach(function (child) {
      return jsList.push(child);
    });
    return ['ol', listStyle].concat(_toConsumableArray(children));
  };
  var RecordFormatter = {
    header: function header(record, config) {
      if (!isRecord(record)) return null;
      var defaults = record.clear();
      var changed = !Immutable.is(defaults, record);
      if (config && config.noPreview) return ['span', changed ? immutableNameStyle : nullStyle, record._name || record.constructor.name || 'Record'];
      var inlinePreview;
      if (!changed) {
        inlinePreview = ['span', inlineValuesStyle, '{}'];
      } else {
        var preview = getKeySeq(record).reduce(function (preview, key) {
          if (Immutable.is(defaults.get(key), record.get(key))) return preview;
          if (preview.length) preview.push(', ');
          preview.push(['span', {}, ['span', keyStyle, key + ': '], reference(record.get(key), {
            noPreview: true
          })]);
          return preview;
        }, []);
        inlinePreview = ['span', inlineValuesStyle, '{'].concat(_toConsumableArray(preview), ['}']);
      }
      return ['span', {}, ['span', immutableNameStyle, record._name || record.constructor.name || 'Record'], ' ', inlinePreview];
    },
    hasBody: hasBody,
    body: function body(record) {
      var defaults = record.clear();
      var children = getKeySeq(record).toJS().map(function (key) {
        var style = Immutable.is(defaults.get(key), record.get(key)) ? defaultValueKeyStyle : alteredValueKeyStyle;
        return ['li', {}, ['span', style, key + ': '], reference(record.get(key))];
      });
      return ['ol', listStyle].concat(_toConsumableArray(children));
    }
  };
  var ListFormatter = {
    header: function header(o) {
      if (!Immutable.isList(o)) return null;
      return renderIterableHeader(o, 'List');
    },
    hasBody: hasBody,
    body: function body(o) {
      return renderIterableBody(o, function (value, key) {
        return ['li', ['span', keyStyle, key + ': '], reference(value)];
      });
    }
  };
  var StackFormatter = {
    header: function header(o) {
      if (!Immutable.isStack(o)) return null;
      return renderIterableHeader(o, 'Stack');
    },
    hasBody: hasBody,
    body: function body(o) {
      return renderIterableBody(o, function (value, key) {
        return ['li', ['span', keyStyle, key + ': '], reference(value)];
      });
    }
  };
  var MapFormatter = {
    header: function header(o) {
      if (!Immutable.isMap(o)) return null;
      return renderIterableHeader(o, 'Map');
    },
    hasBody: hasBody,
    body: function body(o) {
      return renderIterableBody(o, function (value, key) {
        return ['li', {}, '{', reference(key), ' => ', reference(value), '}'];
      }, {
        sorted: true
      });
    }
  };
  var OrderedMapFormatter = {
    header: function header(o) {
      if (!Immutable.isOrderedMap(o)) return null;
      return renderIterableHeader(o, 'OrderedMap');
    },
    hasBody: hasBody,
    body: function body(o) {
      return renderIterableBody(o, function (value, key) {
        return ['li', {}, '{', reference(key), ' => ', reference(value), '}'];
      });
    }
  };
  var SetFormatter = {
    header: function header(o) {
      if (!Immutable.isSet(o)) return null;
      return renderIterableHeader(o, 'Set');
    },
    hasBody: hasBody,
    body: function body(o) {
      return renderIterableBody(o, function (value) {
        return ['li', reference(value)];
      }, {
        sorted: true
      });
    }
  };
  var OrderedSetFormatter = {
    header: function header(o) {
      if (!Immutable.isOrderedSet(o)) return null;
      return renderIterableHeader(o, 'OrderedSet');
    },
    hasBody: hasBody,
    body: function body(o) {
      return renderIterableBody(o, function (value) {
        return ['li', reference(value)];
      });
    }
  };
  var RangeFormatter = {
    header: function header(o) {
      if (!Immutable.isSeq(o)) return null;
      if (typeof o._start === 'undefined' || typeof o._end === 'undefined' || typeof o._step === 'undefined' || typeof o.toString !== 'function') {
        return null;
      }
      var out = o.toString().replace(/^Range /, '');
      return ['span', ['span', immutableNameStyle, 'Range'], ['span', out]];
    },
    hasBody: function hasBody() {
      return false;
    }
  };
  return {
    RecordFormatter: RecordFormatter,
    OrderedMapFormatter: OrderedMapFormatter,
    OrderedSetFormatter: OrderedSetFormatter,
    ListFormatter: ListFormatter,
    MapFormatter: MapFormatter,
    SetFormatter: SetFormatter,
    StackFormatter: StackFormatter,
    RangeFormatter: RangeFormatter
  };
}

// TypeScript entry point for devtools package

// Check for globally defined Immutable and add an install method to it.

if (typeof Immutable !== 'undefined') {
  Immutable.installDevTools = install.bind(null, Immutable);
}
var installed = false;
function install(Immutable) {
  var gw = typeof window === 'undefined' ? global : window;

  // Don't install more than once.
  if (installed === true) {
    return;
  }
  gw.devtoolsFormatters = gw.devtoolsFormatters || [];
  var _createFormatters = createFormatters(Immutable),
    RecordFormatter = _createFormatters.RecordFormatter,
    OrderedMapFormatter = _createFormatters.OrderedMapFormatter,
    OrderedSetFormatter = _createFormatters.OrderedSetFormatter,
    ListFormatter = _createFormatters.ListFormatter,
    MapFormatter = _createFormatters.MapFormatter,
    SetFormatter = _createFormatters.SetFormatter,
    StackFormatter = _createFormatters.StackFormatter,
    RangeFormatter = _createFormatters.RangeFormatter;
  gw.devtoolsFormatters.push(RecordFormatter, OrderedMapFormatter, OrderedSetFormatter, ListFormatter, MapFormatter, SetFormatter, StackFormatter, RangeFormatter);
  installed = true;
}

module.exports = install;
